"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.normalize = exports.genereteSeedAsync = exports.genereteSeed = exports.genereteWords = exports.generateKeyPair = exports.deriveChecksumBits = void 0;
const tweetnacl_1 = __importDefault(require("tweetnacl"));
const hmac_1 = require("@noble/hashes/hmac");
const sha512_1 = require("@noble/hashes/sha512");
const pbkdf2_1 = require("@noble/hashes/pbkdf2");
const hash_1 = require("../utils/hash");
const bip_0039_en_json_1 = __importDefault(require("./bip-0039-en.json"));
const helpers_1 = require("../utils/helpers");
const deriveChecksumBits = (entropy) => {
    const CS = (entropy.length * 8) / 32;
    const hex = (0, hash_1.sha256)(entropy);
    const bits = (0, helpers_1.hexToBits)(hex);
    return bits.slice(0, CS);
};
exports.deriveChecksumBits = deriveChecksumBits;
const generateKeyPair = (seed) => {
    const pair = tweetnacl_1.default.sign.keyPair.fromSeed(seed);
    return {
        private: pair.secretKey,
        public: pair.publicKey
    };
};
exports.generateKeyPair = generateKeyPair;
const genereteWords = () => {
    const entropy = tweetnacl_1.default.randomBytes(32);
    const checkSumBits = deriveChecksumBits(entropy);
    const entropyBits = (0, helpers_1.bytesToBits)(entropy);
    const fullBits = entropyBits.concat(checkSumBits);
    const chunks = fullBits.join('').match(/(.{1,11})/g);
    const words = chunks.map((chunk) => {
        const index = parseInt(chunk, 2);
        return bip_0039_en_json_1.default[index];
    });
    return words;
};
exports.genereteWords = genereteWords;
const genereteSeed = (mnemonic, salt, rounds, keyLength) => {
    const optipns = { c: rounds, dkLen: keyLength };
    const entropy = (0, hmac_1.hmac)(sha512_1.sha512, normalize(mnemonic.join(' ')), '');
    const bytes = (0, pbkdf2_1.pbkdf2)(sha512_1.sha512, entropy, salt, optipns);
    return bytes;
};
exports.genereteSeed = genereteSeed;
const genereteSeedAsync = async (mnemonic, salt, rounds, keyLength) => {
    const optipns = { c: rounds, dkLen: keyLength };
    const entropy = (0, hmac_1.hmac)(sha512_1.sha512, normalize(mnemonic.join(' ')), '');
    const bytes = await (0, pbkdf2_1.pbkdf2Async)(sha512_1.sha512, entropy, salt, optipns);
    return bytes;
};
exports.genereteSeedAsync = genereteSeedAsync;
const normalize = (value) => {
    return (value || '').normalize('NFKD');
};
exports.normalize = normalize;
//# sourceMappingURL=utlis.js.map